@@ -13,45 +13,56 @@
 #include <QMimeData>
 #include <QMutex>
 
-#include <KSharedConfig>
 #include <KLocalizedString>
+#include <KSharedConfig>
 #if KRUNNER_BUILD_DEPRECATED_SINCE(5, 65)
 #include <Plasma/Package>
 #endif
 
 #include "krunner_debug.h"
-#include "querymatch.h"
-#include "runnercontext.h"
 
 namespace Plasma
 {
-
+#if KRUNNER_BUILD_DEPRECATED_SINCE(5, 77)
 AbstractRunner::AbstractRunner(QObject *parent, const QString &path)
-    : QObject(parent),
-      d(new AbstractRunnerPrivate(this))
+    : QObject(parent)
+    , d(new AbstractRunnerPrivate(this))
 {
     d->init(path);
 }
+#endif
 
+#if KRUNNER_BUILD_DEPRECATED_SINCE(5, 86)
 AbstractRunner::AbstractRunner(const KPluginMetaData &pluginMetaData, QObject *parent)
-    : QObject(parent),
-      d(new AbstractRunnerPrivate(this))
+    : QObject(parent)
+    , d(new AbstractRunnerPrivate(this))
 {
     d->init(pluginMetaData);
 }
+#endif
 
 #if KRUNNER_BUILD_DEPRECATED_SINCE(5, 72) && KSERVICE_BUILD_DEPRECATED_SINCE(5, 0)
 AbstractRunner::AbstractRunner(const KService::Ptr service, QObject *parent)
-    : QObject(parent),
-      d(new AbstractRunnerPrivate(this))
+    : QObject(parent)
+    , d(new AbstractRunnerPrivate(this))
 {
     d->init(service);
 }
 #endif
 
+AbstractRunner::AbstractRunner(QObject *parent, const KPluginMetaData &pluginMetaData, const QVariantList &args)
+    : QObject(parent)
+    , d(new AbstractRunnerPrivate(this))
+{
+    Q_UNUSED(args)
+
+    d->init(pluginMetaData);
+}
+
+#if KRUNNER_BUILD_DEPRECATED_SINCE(5, 77)
 AbstractRunner::AbstractRunner(QObject *parent, const QVariantList &args)
-    : QObject(parent),
-      d(new AbstractRunnerPrivate(this))
+    : QObject(parent)
+    , d(new AbstractRunnerPrivate(this))
 {
     if (!args.isEmpty()) {
 #if KRUNNER_BUILD_DEPRECATED_SINCE(5, 72) && KSERVICE_BUILD_DEPRECATED_SINCE(5, 0)
@@ -60,7 +71,7 @@ AbstractRunner::AbstractRunner(QObject *parent, const QVariantList &args)
         if (args.size() > 1) {
             const KPluginMetaData metaData = args[1].value<KPluginMetaData>();
 #else
-            const KPluginMetaData metaData = args[0].value<KPluginMetaData>();
+        const KPluginMetaData metaData = args[0].value<KPluginMetaData>();
 #endif
             if (metaData.isValid()) {
                 d->init(metaData);
@@ -76,11 +87,9 @@ AbstractRunner::AbstractRunner(QObject *parent, const QVariantList &args)
 #endif
     }
 }
+#endif
 
-AbstractRunner::~AbstractRunner()
-{
-    delete d;
-}
+AbstractRunner::~AbstractRunner() = default;
 
 KConfigGroup AbstractRunner::config() const
 {
@@ -102,11 +111,23 @@ void AbstractRunner::addSyntax(const RunnerSyntax &syntax)
     d->syntaxes.append(syntax);
 }
 
+bool AbstractRunner::hasUniqueResults()
+{
+    return d->hasUniqueResults;
+}
+
+bool AbstractRunner::hasWeakResults()
+{
+    return d->hasWeakResults;
+}
+
+#if KRUNNER_BUILD_DEPRECATED_SINCE(5, 76)
 void AbstractRunner::setDefaultSyntax(const RunnerSyntax &syntax)
 {
     d->syntaxes.append(syntax);
     d->defaultSyntax = &(d->syntaxes.last());
 }
+#endif
 
 void AbstractRunner::setSyntaxes(const QList<RunnerSyntax> &syntaxes)
 {
@@ -118,24 +139,23 @@ QList<RunnerSyntax> AbstractRunner::syntaxes() const
     return d->syntaxes;
 }
 
+#if KRUNNER_BUILD_DEPRECATED_SINCE(5, 76)
 RunnerSyntax *AbstractRunner::defaultSyntax() const
 {
     return d->defaultSyntax;
 }
+#endif
 
+#if KRUNNER_BUILD_DEPRECATED_SINCE(5, 81)
 void AbstractRunner::performMatch(Plasma::RunnerContext &localContext)
 {
     static const int reasonableRunTime = 1500;
     static const int fastEnoughTime = 250;
 
-    if (d->suspendMatching) {
-        return;
-    }
-
     QElapsedTimer time;
     time.start();
 
-    //The local copy is already obtained in the job
+    // The local copy is already obtained in the job
     match(localContext);
 
     // automatically rate limit runners that become slooow
@@ -159,13 +179,15 @@ void AbstractRunner::performMatch(Plasma::RunnerContext &localContext)
         }
     }
 }
+#endif
 
-QList<QAction*> AbstractRunner::actionsForMatch(const Plasma::QueryMatch &match)
+QList<QAction *> AbstractRunner::actionsForMatch(const Plasma::QueryMatch &match)
 {
     return match.isValid() ? match.actions() : QList<QAction *>();
 }
 
-QAction* AbstractRunner::addAction(const QString &id, const QIcon &icon, const QString &text)
+#if KRUNNER_BUILD_DEPRECATED_SINCE(5, 86)
+QAction *AbstractRunner::addAction(const QString &id, const QIcon &icon, const QString &text)
 {
     QAction *a = new QAction(icon, text, this);
     d->actions.insert(id, a);
@@ -183,12 +205,12 @@ void AbstractRunner::removeAction(const QString &id)
     delete a;
 }
 
-QAction* AbstractRunner::action(const QString &id) const
+QAction *AbstractRunner::action(const QString &id) const
 {
     return d->actions.value(id);
 }
 
-QHash<QString, QAction*> AbstractRunner::actions() const
+QHash<QString, QAction *> AbstractRunner::actions() const
 {
     return d->actions;
 }
@@ -198,6 +220,7 @@ void AbstractRunner::clearActions()
     qDeleteAll(d->actions);
     d->actions.clear();
 }
+#endif
 
 QMimeData *AbstractRunner::mimeDataForMatch(const QueryMatch &match)
 {
@@ -230,6 +253,7 @@ void AbstractRunner::createRunOptions(QWidget *parent)
 }
 #endif
 
+#if KRUNNER_BUILD_DEPRECATED_SINCE(5, 81)
 AbstractRunner::Speed AbstractRunner::speed() const
 {
     // the only time the read lock will fail is if we were slow are going to speed up
@@ -243,13 +267,16 @@ AbstractRunner::Speed AbstractRunner::speed() const
     d->speedLock.unlock();
     return s;
 }
+#endif
 
+#if KRUNNER_BUILD_DEPRECATED_SINCE(5, 81)
 void AbstractRunner::setSpeed(Speed speed)
 {
     d->speedLock.lockForWrite();
     d->speed = speed;
     d->speedLock.unlock();
 }
+#endif
 
 AbstractRunner::Priority AbstractRunner::priority() const
 {
@@ -261,15 +288,19 @@ void AbstractRunner::setPriority(Priority priority)
     d->priority = priority;
 }
 
+#if KRUNNER_BUILD_DEPRECATED_SINCE(5, 76)
 RunnerContext::Types AbstractRunner::ignoredTypes() const
 {
     return d->blackListed;
 }
+#endif
 
+#if KRUNNER_BUILD_DEPRECATED_SINCE(5, 76)
 void AbstractRunner::setIgnoredTypes(RunnerContext::Types types)
 {
     d->blackListed = types;
 }
+#endif
 
 void AbstractRunner::run(const Plasma::RunnerContext &search, const Plasma::QueryMatch &action)
 {
@@ -277,15 +308,19 @@ void AbstractRunner::run(const Plasma::RunnerContext &search, const Plasma::Quer
     Q_UNUSED(action);
 }
 
+#if KRUNNER_BUILD_DEPRECATED_SINCE(5, 76)
 QStringList AbstractRunner::categories() const
 {
     return QStringList() << name();
 }
+#endif
 
-QIcon AbstractRunner::categoryIcon(const QString&) const
+#if KRUNNER_BUILD_DEPRECATED_SINCE(5, 76)
+QIcon AbstractRunner::categoryIcon(const QString &) const
 {
     return icon();
 }
+#endif
 
 void AbstractRunner::match(Plasma::RunnerContext &)
 {
@@ -342,11 +377,11 @@ KPluginMetaData AbstractRunner::metadata(RunnerReturnPluginMetaDataConstant) con
 #if KRUNNER_BUILD_DEPRECATED_SINCE(5, 65)
 Package AbstractRunner::package() const
 {
-QT_WARNING_PUSH
-QT_WARNING_DISABLE_CLANG("-Wdeprecated-declarations")
-QT_WARNING_DISABLE_GCC("-Wdeprecated-declarations")
+    QT_WARNING_PUSH
+    QT_WARNING_DISABLE_CLANG("-Wdeprecated-declarations")
+    QT_WARNING_DISABLE_GCC("-Wdeprecated-declarations")
     return Package();
-QT_WARNING_POP
+    QT_WARNING_POP
 }
 #endif
 
@@ -425,14 +460,16 @@ bool AbstractRunner::hasMatchRegex() const
 }
 
 AbstractRunnerPrivate::AbstractRunnerPrivate(AbstractRunner *r)
-    : priority(AbstractRunner::NormalPriority),
-      speed(AbstractRunner::NormalSpeed),
-      blackListed(RunnerContext::None),
-      runner(r),
-      fastRuns(0),
-      defaultSyntax(nullptr),
-      hasRunOptions(false),
-      suspendMatching(false)
+    : priority(AbstractRunner::NormalPriority)
+#if KRUNNER_BUILD_DEPRECATED_SINCE(5, 81)
+    , speed(AbstractRunner::NormalSpeed)
+#endif
+    , blackListed(RunnerContext::None)
+    , runner(r)
+    , fastRuns(0)
+    , defaultSyntax(nullptr)
+    , hasRunOptions(false)
+    , suspendMatching(false)
 {
 }
 
@@ -449,6 +486,8 @@ void AbstractRunnerPrivate::init()
             matchRegex = QRegularExpression(rawData.value(QStringLiteral("X-Plasma-Runner-Match-Regex")).toString());
             hasMatchRegex = matchRegex.isValid() && !matchRegex.pattern().isEmpty();
         }
+        hasUniqueResults = rawData.value(QStringLiteral("X-Plasma-Runner-Unique-Results")).toBool();
+        hasWeakResults = rawData.value(QStringLiteral("X-Plasma-Runner-Weak-Results")).toBool();
     }
 }
 
@@ -461,23 +500,24 @@ void AbstractRunnerPrivate::init(const KPluginMetaData &pluginMetaData)
 #if KRUNNER_BUILD_DEPRECATED_SINCE(5, 72) && KSERVICE_BUILD_DEPRECATED_SINCE(5, 0)
 void AbstractRunnerPrivate::init(const KService::Ptr service)
 {
-QT_WARNING_PUSH
-QT_WARNING_DISABLE_CLANG("-Wdeprecated-declarations")
-QT_WARNING_DISABLE_GCC("-Wdeprecated-declarations")
+    QT_WARNING_PUSH
+    QT_WARNING_DISABLE_CLANG("-Wdeprecated-declarations")
+    QT_WARNING_DISABLE_GCC("-Wdeprecated-declarations")
     const KPluginInfo pluginInfo(service);
     runnerDescription = pluginInfo.isValid() ? pluginInfo.toMetaData() : KPluginMetaData();
-QT_WARNING_POP
+    QT_WARNING_POP
     init();
 }
 #endif
 
+#if KRUNNER_BUILD_DEPRECATED_SINCE(5, 77)
 void AbstractRunnerPrivate::init(const QString &path)
 {
     runnerDescription = KPluginMetaData(path + QStringLiteral("/metadata.desktop"));
     init();
 }
+#endif
 
 } // Plasma namespace
 
-
 #include "moc_abstractrunner.cpp"
