@@ -16,6 +16,7 @@
 #include <QTimer>
 
 #include <KConfigWatcher>
+#include <KFileUtils>
 #include <KPluginMetaData>
 #include <KServiceTypeTrader>
 #include <KSharedConfig>
@@ -36,32 +37,10 @@
 #include "querymatch.h"
 #include "runnerjobs_p.h"
 
-using ThreadWeaver::Job;
 using ThreadWeaver::Queue;
 
 namespace Plasma
 {
-void forEachDBusPlugin(std::function<void(const KPluginMetaData &, bool *)> callback)
-{
-    const QStringList dBusPlugindirs =
-        QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, QStringLiteral("krunner/dbusplugins"), QStandardPaths::LocateDirectory);
-    const QStringList serviceTypeFiles(QStringLiteral("plasma-runner.desktop"));
-    for (const QString &dir : dBusPlugindirs) {
-        const QStringList desktopFiles = QDir(dir).entryList(QStringList(QStringLiteral("*.desktop")));
-        for (const QString &file : desktopFiles) {
-            const QString desktopFilePath = dir + QLatin1Char('/') + file;
-            KPluginMetaData pluginMetaData = KPluginMetaData::fromDesktopFile(desktopFilePath, serviceTypeFiles);
-            if (pluginMetaData.isValid()) {
-                bool cancel = false;
-                callback(pluginMetaData, &cancel);
-                if (cancel) {
-                    return;
-                }
-            }
-        }
-    }
-}
-
 #if KSERVICE_BUILD_DEPRECATED_SINCE(5, 0)
 void warnAboutDeprecatedMetaData(const KPluginInfo &pluginInfo)
 {
@@ -75,29 +54,24 @@ void warnAboutDeprecatedMetaData(const KPluginInfo &pluginInfo)
 }
 #endif
 
-/*****************************************************
- *  RunnerManager::Private class
- *
- *****************************************************/
 class RunnerManagerPrivate
 {
 public:
     RunnerManagerPrivate(RunnerManager *parent)
         : q(parent)
-        , deferredRun(nullptr)
-        , currentSingleRunner(nullptr)
-        , prepped(false)
-        , allRunnersPrepped(false)
-        , singleRunnerPrepped(false)
-        , teardownRequested(false)
-        , singleMode(false)
     {
         matchChangeTimer.setSingleShot(true);
         delayTimer.setSingleShot(true);
 
-        QObject::connect(&matchChangeTimer, SIGNAL(timeout()), q, SLOT(matchesChanged()));
-        QObject::connect(&context, SIGNAL(matchesChanged()), q, SLOT(scheduleMatchesChanged()));
-        QObject::connect(&delayTimer, SIGNAL(timeout()), q, SLOT(unblockJobs()));
+        QObject::connect(&matchChangeTimer, &QTimer::timeout, q, [this]() {
+            matchesChanged();
+        });
+        QObject::connect(&context, &RunnerContext::matchesChanged, q, [this]() {
+            scheduleMatchesChanged();
+        });
+        QObject::connect(&delayTimer, &QTimer::timeout, q, [this]() {
+            unblockJobs();
+        });
 
         // Set up tracking of the last time matchesChanged was signalled
         lastMatchChangeSignalled.start();
@@ -163,13 +137,16 @@ public:
         if (currentSingleRunner && currentSingleRunner->id() == singleModeRunnerId) {
             return;
         }
-        if (runners.isEmpty()) {
-            loadRunners();
-        }
         currentSingleRunner = q->runner(singleModeRunnerId);
+        // If there are no runners loaded or the single runner could no be loaded,
+        // this is the case if it was disabled but gets queries using the singleRunnerMode, BUG: 435050
+        if (runners.isEmpty() || !currentSingleRunner) {
+            loadRunners(singleModeRunnerId);
+            currentSingleRunner = q->runner(singleModeRunnerId);
+        }
     }
 
-    void loadRunners()
+    void loadRunners(const QString &singleRunnerId = QString())
     {
         QVector<KPluginMetaData> offers = RunnerManager::runnerMetaDataList();
 
@@ -177,30 +154,30 @@ public:
         const bool noWhiteList = whiteList.isEmpty();
         KConfigGroup pluginConf = configPrt->group("Plugins");
 
-        advertiseSingleRunnerIds.clear();
-
-        QStringList allCategories;
         QSet<AbstractRunner *> deadRunners;
         QMutableVectorIterator<KPluginMetaData> it(offers);
         while (it.hasNext()) {
             KPluginMetaData &description = it.next();
             qCDebug(KRUNNER) << "Loading runner: " << description.pluginId();
 
+#if KRUNNER_BUILD_DEPRECATED_SINCE(5, 85)
             const QString tryExec = description.value(QStringLiteral("TryExec"));
-            if (!tryExec.isEmpty() && QStandardPaths::findExecutable(tryExec).isEmpty()) {
-                // we don't actually have this application!
-                continue;
+            if (!tryExec.isEmpty()) {
+                qCDebug(KRUNNER) << description.pluginId() << "The TryExec property is deprecated, manually check if the application exists if needed";
+                if (QStandardPaths::findExecutable(tryExec).isEmpty()) {
+                    // we don't actually have this application!
+                    continue;
+                }
             }
+#endif
 
             const QString runnerName = description.pluginId();
-            const bool isPluginEnabled = pluginConf.readEntry(runnerName + QLatin1String("Enabled"), description.isEnabledByDefault());
+            const bool isPluginEnabled = description.isEnabled(pluginConf);
             const bool loaded = runners.contains(runnerName);
-            const bool selected = loadAll || (isPluginEnabled && (noWhiteList || whiteList.contains(runnerName)));
-
-            const bool singleQueryModeEnabled = description.rawData().value(QStringLiteral("X-Plasma-AdvertiseSingleRunnerQueryMode")).toVariant().toBool();
-
-            if (singleQueryModeEnabled) {
-                advertiseSingleRunnerIds.insert(runnerName, description.name());
+            bool selected = loadAll || disabledRunnerIds.contains(runnerName) || (isPluginEnabled && (noWhiteList || whiteList.contains(runnerName)));
+            if (!selected && runnerName == singleRunnerId) {
+                selected = true;
+                disabledRunnerIds << runnerName;
             }
 
             if (selected) {
@@ -215,7 +192,6 @@ public:
                     bool allCategoriesDisabled = true;
 #if KRUNNER_BUILD_DEPRECATED_SINCE(5, 76)
                     const QStringList categories = runner->categories();
-                    allCategories << categories;
 
                     for (const QString &cat : categories) {
                         if (enabledCategories.contains(cat)) {
@@ -241,17 +217,13 @@ public:
             }
         }
 
-        if (enabledCategories.isEmpty()) {
-            enabledCategories = allCategories;
-        }
-
         if (!deadRunners.isEmpty()) {
             QSet<QSharedPointer<FindMatchesJob>> deadJobs;
             auto it = searchJobs.begin();
             while (it != searchJobs.end()) {
                 auto &job = (*it);
                 if (deadRunners.contains(job->runner())) {
-                    QObject::disconnect(job.data(), SIGNAL(done(ThreadWeaver::JobPointer)), q, SLOT(jobDone(ThreadWeaver::JobPointer)));
+                    QObject::disconnect(job.data(), &FindMatchesJob::done, q, nullptr);
                     it = searchJobs.erase(it);
                     deadJobs.insert(job);
                 } else {
@@ -294,38 +266,33 @@ public:
         const QString api = pluginMetaData.value(QStringLiteral("X-Plasma-API"));
 
         if (api.isEmpty()) {
-            KPluginLoader pluginLoader(pluginMetaData.fileName());
-            const quint64 pluginVersion = pluginLoader.pluginVersion();
-            if (Plasma::isPluginVersionCompatible(pluginVersion)) {
-                if (KPluginFactory *factory = pluginLoader.factory()) {
-                    const QVariantList args
-                    {
+                const QVariantList args
+                {
 #if KRUNNER_BUILD_DEPRECATED_SINCE(5, 77)
 #if KSERVICE_BUILD_DEPRECATED_SINCE(5, 0)
-                        pluginMetaData.metaDataFileName(),
+                    pluginMetaData.metaDataFileName(),
 #endif
-                            QVariant::fromValue(pluginMetaData),
+                        QVariant::fromValue(pluginMetaData),
 #endif
-                    };
-                    runner = factory->create<AbstractRunner>(q, args);
+                };
+                auto res = KPluginFactory::instantiatePlugin<AbstractRunner>(pluginMetaData, q, args);
+                if (res) {
+                    runner = res.plugin;
                 } else {
-                    qCWarning(KRUNNER).nospace() << "Could not load runner " << pluginMetaData.name() << ":" << pluginLoader.errorString()
+                    qCWarning(KRUNNER).nospace() << "Could not load runner " << pluginMetaData.name() << ":" << res.errorString
                                                  << " (library path was:" << pluginMetaData.fileName() << ")";
                 }
-            } else {
-                const QString runnerVersion =
-                    QStringLiteral("%1.%2.%3").arg(pluginVersion >> 16).arg((pluginVersion >> 8) & 0x00ff).arg(pluginVersion & 0x0000ff);
-                qCWarning(KRUNNER) << "Cannot load runner" << pluginMetaData.name() << "- versions mismatch: KRunner" << Plasma::versionString() << ","
-                                   << pluginMetaData.name() << runnerVersion;
-            }
-        } else if (api == QLatin1String("DBus")) {
-            runner = new DBusRunner(pluginMetaData, q);
+        } else if (api.startsWith(QLatin1String("DBus"))) {
+            runner = new DBusRunner(q, pluginMetaData, {});
         } else {
-            runner = new AbstractRunner(pluginMetaData, q);
+            qCWarning(KRUNNER) << "Unknown X-Plasma-API requested for runner" << pluginMetaData.fileName();
+            return nullptr;
         }
 
         if (runner) {
-            QObject::connect(runner, SIGNAL(matchingSuspended(bool)), q, SLOT(runnerMatchingSuspended(bool)));
+            QObject::connect(runner, &AbstractRunner::matchingSuspended, q, [this](bool state) {
+                runnerMatchingSuspended(state);
+            });
             runner->init();
             if (prepped) {
                 Q_EMIT runner->prepare();
@@ -343,12 +310,6 @@ public:
             return;
         }
 
-        if (deferredRun.isEnabled() && runJob->runner() == deferredRun.runner()) {
-            QueryMatch tmpRun = deferredRun;
-            deferredRun = QueryMatch(nullptr);
-            tmpRun.run(context);
-        }
-
         searchJobs.remove(runJob);
         oldSearchJobs.remove(runJob);
 
@@ -376,7 +337,7 @@ public:
 
         if (searchJobs.isEmpty() && oldSearchJobs.isEmpty()) {
             if (allRunnersPrepped) {
-                for (AbstractRunner *runner : qAsConst(runners)) {
+                for (AbstractRunner *runner : std::as_const(runners)) {
                     Q_EMIT runner->teardown();
                 }
 
@@ -409,12 +370,16 @@ public:
 
     void runnerMatchingSuspended(bool suspended)
     {
-        if (suspended || !prepped || teardownRequested) {
+        auto *runner = qobject_cast<AbstractRunner *>(q->sender());
+        if (suspended || !prepped || teardownRequested || !runner) {
             return;
         }
 
-        if (auto *runner = qobject_cast<AbstractRunner *>(q->sender())) {
-            startJob(runner);
+        const QString query = context.query();
+        if (singleMode || runner->minLetterCount() <= query.size()) {
+            if (singleMode || !runner->hasMatchRegex() || runner->matchRegex().match(query).hasMatch()) {
+                startJob(runner);
+            }
         }
     }
 
@@ -426,7 +391,9 @@ public:
         }
 #endif
         QSharedPointer<FindMatchesJob> job(new FindMatchesJob(runner, &context, Queue::instance()));
-        QObject::connect(job.data(), SIGNAL(done(ThreadWeaver::JobPointer)), q, SLOT(jobDone(ThreadWeaver::JobPointer)));
+        QObject::connect(job.data(), &FindMatchesJob::done, q, [this](ThreadWeaver::JobPointer jobPtr) {
+            jobDone(jobPtr);
+        });
 
 #if KRUNNER_BUILD_DEPRECATED_SINCE(5, 81)
         if (runner->speed() == AbstractRunner::SlowSpeed) {
@@ -489,7 +456,7 @@ public:
             historyEntries.removeOne(a);
         }
 
-        for (const QString &deletedActivity : qAsConst(historyEntries)) {
+        for (const QString &deletedActivity : std::as_const(historyEntries)) {
             historyGroup.deleteEntry(deletedActivity);
         }
         historyGroup.sync();
@@ -505,43 +472,37 @@ public:
     static const int slowRunDelay = 400;
 
     RunnerManager *const q;
-    QueryMatch deferredRun;
     RunnerContext context;
     QTimer matchChangeTimer;
     QTimer delayTimer; // Timer to control when to run slow runners
     QElapsedTimer lastMatchChangeSignalled;
     QHash<QString, AbstractRunner *> runners;
-    QHash<QString, QString> advertiseSingleRunnerIds;
-    AbstractRunner *currentSingleRunner;
+    AbstractRunner *currentSingleRunner = nullptr;
     QSet<QSharedPointer<FindMatchesJob>> searchJobs;
     QSet<QSharedPointer<FindMatchesJob>> oldSearchJobs;
     QStringList enabledCategories;
     QString singleModeRunnerId;
-    bool prepped : 1;
-    bool allRunnersPrepped : 1;
-    bool singleRunnerPrepped : 1;
-    bool teardownRequested : 1;
-    bool singleMode : 1;
-    bool activityAware : 1;
-    bool historyEnabled : 1;
-    bool retainPriorSearch : 1;
+    bool prepped = false;
+    bool allRunnersPrepped = false;
+    bool singleRunnerPrepped = false;
+    bool teardownRequested = false;
+    bool singleMode = false;
+    bool activityAware = false;
+    bool historyEnabled = false;
+    bool retainPriorSearch = false;
     QStringList whiteList;
-    QString configFile;
     KConfigWatcher::Ptr watcher;
     QHash<QString, QString> priorSearch;
     QString untrimmedTerm;
     QString nulluuid = QStringLiteral("00000000-0000-0000-0000-000000000000");
     KSharedConfigPtr configPrt;
     KConfigGroup stateData;
+    QSet<QString> disabledRunnerIds; // Runners that are disabled but were loaded as single runners
 #ifdef HAVE_KACTIVITIES
     const KActivities::Consumer activitiesConsumer;
 #endif
 };
 
-/*****************************************************
- *  RunnerManager::Public class
- *
- *****************************************************/
 RunnerManager::RunnerManager(QObject *parent)
     : RunnerManager(QString(), parent)
 {
@@ -580,8 +541,6 @@ RunnerManager::~RunnerManager()
     if (!qApp->closingDown() && (!d->searchJobs.isEmpty() || !d->oldSearchJobs.isEmpty())) {
         new DelayedJobCleaner(d->searchJobs + d->oldSearchJobs);
     }
-
-    delete d;
 }
 
 void RunnerManager::reloadConfiguration()
@@ -613,10 +572,12 @@ void RunnerManager::setEnabledCategories(const QStringList &categories)
 }
 #endif
 
+#if KRUNNER_BUILD_DEPRECATED_SINCE(5, 88)
 QStringList RunnerManager::allowedRunners() const
 {
     return d->stateData.readEntry("pluginWhiteList", QStringList());
 }
+#endif
 
 #if KRUNNER_BUILD_DEPRECATED_SINCE(5, 76)
 QStringList RunnerManager::enabledCategories() const
@@ -624,7 +585,7 @@ QStringList RunnerManager::enabledCategories() const
     QStringList list = d->stateData.readEntry("enabledCategories", QStringList());
     if (list.isEmpty()) {
         list.reserve(d->runners.count());
-        for (AbstractRunner *runner : qAsConst(d->runners)) {
+        for (AbstractRunner *runner : std::as_const(d->runners)) {
             list << runner->categories();
         }
     }
@@ -660,7 +621,9 @@ void RunnerManager::loadRunner(const QString &path)
 {
     if (!d->runners.contains(path)) {
         AbstractRunner *runner = new AbstractRunner(this, path);
-        connect(runner, SIGNAL(matchingSuspended(bool)), this, SLOT(runnerMatchingSuspended(bool)));
+        connect(runner, &AbstractRunner::matchingSuspended, this, [this](bool state) {
+            d->runnerMatchingSuspended(state);
+        });
         d->runners.insert(path, runner);
     }
 }
@@ -675,6 +638,7 @@ AbstractRunner *RunnerManager::runner(const QString &name) const
     return d->runners.value(name, nullptr);
 }
 
+#if KRUNNER_BUILD_DEPRECATED_SINCE(5, 82)
 AbstractRunner *RunnerManager::singleModeRunner() const
 {
     return d->currentSingleRunner;
@@ -718,23 +682,26 @@ void RunnerManager::setSingleMode(bool singleMode)
     }
 }
 
-QList<AbstractRunner *> RunnerManager::runners() const
-{
-    return d->runners.values();
-}
-
 QStringList RunnerManager::singleModeAdvertisedRunnerIds() const
 {
-    return d->advertiseSingleRunnerIds.keys();
+    QStringList advertiseSingleRunnerIds;
+    for (auto *runner : std::as_const(d->runners)) {
+        if (runner->metadata(RunnerReturnPluginMetaData).rawData().value(QStringLiteral("X-Plasma-AdvertiseSingleRunnerQueryMode")).toVariant().toBool()) {
+            advertiseSingleRunnerIds << runner->id();
+        }
+    }
+    return advertiseSingleRunnerIds;
 }
 
 QString RunnerManager::runnerName(const QString &id) const
 {
-    if (runner(id)) {
-        return runner(id)->name();
-    } else {
-        return d->advertiseSingleRunnerIds.value(id, QString());
-    }
+    return d->runners.contains(id) ? d->runners.value(id)->name() : QString();
+}
+#endif
+
+QList<AbstractRunner *> RunnerManager::runners() const
+{
+    return d->runners.values();
 }
 
 RunnerContext *RunnerManager::searchContext() const
@@ -757,25 +724,9 @@ void RunnerManager::run(const QString &matchId)
 
 void RunnerManager::run(const QueryMatch &match)
 {
-    if (!match.isEnabled()) {
-        return;
+    if (match.isEnabled()) {
+        d->context.run(match);
     }
-
-    // TODO: this function is not const as it may be used for learning
-    AbstractRunner *runner = match.runner();
-
-    for (auto it = d->searchJobs.constBegin(); it != d->searchJobs.constEnd(); ++it) {
-        if ((*it)->runner() == runner && !(*it)->isFinished()) {
-            d->deferredRun = match;
-            return;
-        }
-    }
-
-    if (d->deferredRun.isValid()) {
-        d->deferredRun = QueryMatch(nullptr);
-    }
-
-    d->context.run(match);
 }
 
 bool RunnerManager::runMatch(const QueryMatch &match)
@@ -813,6 +764,7 @@ QMimeData *RunnerManager::mimeDataForMatch(const QueryMatch &match) const
     return match.isValid() ? match.runner()->mimeDataForMatch(match) : nullptr;
 }
 
+#if KRUNNER_BUILD_DEPRECATED_SINCE(5, 85)
 QVector<KPluginMetaData> RunnerManager::runnerMetaDataList(const QString &parentApp)
 {
     // get binary plugins
@@ -821,18 +773,23 @@ QVector<KPluginMetaData> RunnerManager::runnerMetaDataList(const QString &parent
         return md.value(QStringLiteral("X-KDE-ParentApp")) == parentApp;
     };
 
-    QVector<KPluginMetaData> pluginMetaDatas = KPluginLoader::findPlugins(QStringLiteral("kf5/krunner"), filterParentApp);
+    QVector<KPluginMetaData> pluginMetaDatas = KPluginMetaData::findPlugins(QStringLiteral("kf5/krunner"), filterParentApp);
     QSet<QString> knownRunnerIds;
     knownRunnerIds.reserve(pluginMetaDatas.size());
-    for (const KPluginMetaData &pluginMetaData : qAsConst(pluginMetaDatas)) {
+    for (const KPluginMetaData &pluginMetaData : std::as_const(pluginMetaDatas)) {
         knownRunnerIds.insert(pluginMetaData.pluginId());
     }
 
-    // get D-Bus plugins
-    forEachDBusPlugin([&](const KPluginMetaData &pluginMetaData, bool *) {
-        pluginMetaDatas.append(pluginMetaData);
-        knownRunnerIds.insert(pluginMetaData.pluginId());
-    });
+    const QStringList dBusPlugindirs =
+        QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, QStringLiteral("krunner/dbusplugins"), QStandardPaths::LocateDirectory);
+    const QStringList dbusRunnerFiles = KFileUtils::findAllUniqueFiles(dBusPlugindirs, QStringList(QStringLiteral("*.desktop")));
+    for (const QString &dbusRunnerFile : dbusRunnerFiles) {
+        KPluginMetaData pluginMetaData = KPluginMetaData::fromDesktopFile(dbusRunnerFile, QStringList(QStringLiteral("plasma-runner.desktop")));
+        if (pluginMetaData.isValid() && !knownRunnerIds.contains(pluginMetaData.pluginId())) {
+            pluginMetaDatas.append(pluginMetaData);
+            knownRunnerIds.insert(pluginMetaData.pluginId());
+        }
+    }
 
 #if KSERVICE_BUILD_DEPRECATED_SINCE(5, 0)
     // also search for deprecated kservice-based KRunner plugins metadata
@@ -856,6 +813,47 @@ QVector<KPluginMetaData> RunnerManager::runnerMetaDataList(const QString &parent
 
     return pluginMetaDatas;
 }
+#endif
+
+QVector<KPluginMetaData> RunnerManager::runnerMetaDataList()
+{
+    QVector<KPluginMetaData> pluginMetaDatas = KPluginMetaData::findPlugins(QStringLiteral("kf5/krunner"));
+    QSet<QString> knownRunnerIds;
+    knownRunnerIds.reserve(pluginMetaDatas.size());
+    for (const KPluginMetaData &pluginMetaData : std::as_const(pluginMetaDatas)) {
+        knownRunnerIds.insert(pluginMetaData.pluginId());
+    }
+
+    const QStringList dBusPlugindirs =
+        QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, QStringLiteral("krunner/dbusplugins"), QStandardPaths::LocateDirectory);
+    const QStringList dbusRunnerFiles = KFileUtils::findAllUniqueFiles(dBusPlugindirs, QStringList(QStringLiteral("*.desktop")));
+    for (const QString &dbusRunnerFile : dbusRunnerFiles) {
+        KPluginMetaData pluginMetaData = KPluginMetaData::fromDesktopFile(dbusRunnerFile, QStringList(QStringLiteral("plasma-runner.desktop")));
+        if (pluginMetaData.isValid() && !knownRunnerIds.contains(pluginMetaData.pluginId())) {
+            pluginMetaDatas.append(pluginMetaData);
+            knownRunnerIds.insert(pluginMetaData.pluginId());
+        }
+    }
+
+#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 0)
+    // also search for deprecated kservice-based KRunner plugins metadata
+    const KService::List offers = KServiceTypeTrader::self()->query(QStringLiteral("Plasma/Runner"));
+    QT_WARNING_PUSH
+    QT_WARNING_DISABLE_CLANG("-Wdeprecated-declarations")
+    QT_WARNING_DISABLE_GCC("-Wdeprecated-declarations")
+    const KPluginInfo::List backwardCompatPluginInfos = KPluginInfo::fromServices(offers);
+    QT_WARNING_POP
+
+    for (const KPluginInfo &pluginInfo : backwardCompatPluginInfos) {
+        if (!knownRunnerIds.contains(pluginInfo.pluginName())) {
+            warnAboutDeprecatedMetaData(pluginInfo);
+            pluginMetaDatas.append(pluginInfo.toMetaData());
+        }
+    }
+#endif
+
+    return pluginMetaDatas;
+}
 
 #if KRUNNER_BUILD_DEPRECATED_SINCE(5, 72)
 KPluginInfo::List RunnerManager::listRunnerInfo(const QString &parentApp)
@@ -879,8 +877,10 @@ void RunnerManager::setupMatchSession()
             d->singleRunnerPrepped = true;
         }
     } else {
-        for (AbstractRunner *runner : qAsConst(d->runners)) {
-            Q_EMIT runner->prepare();
+        for (AbstractRunner *runner : std::as_const(d->runners)) {
+            if (!d->disabledRunnerIds.contains(runner->name())) {
+                Q_EMIT runner->prepare();
+            }
         }
 
         d->allRunnersPrepped = true;
@@ -909,16 +909,20 @@ void RunnerManager::launchQuery(const QString &untrimmedTerm, const QString &run
 {
     setupMatchSession();
     QString term = untrimmedTerm.trimmed();
+    const QString prevSingleRunner = d->singleModeRunnerId;
     d->untrimmedTerm = untrimmedTerm;
 
-    setSingleModeRunnerId(runnerName);
-    setSingleMode(d->currentSingleRunner);
+    // Set the required values and load the runner
+    d->singleModeRunnerId = runnerName;
+    d->singleMode = !runnerName.isEmpty();
+    d->loadSingleRunner();
+    // If we could not load the single runner we reset
     if (!runnerName.isEmpty() && !d->currentSingleRunner) {
         reset();
         return;
     }
 
-    if (d->context.query() == term) {
+    if (d->context.query() == term && prevSingleRunner == runnerName) {
         // we already are searching for this!
         return;
     }
@@ -944,10 +948,14 @@ void RunnerManager::launchQuery(const QString &untrimmedTerm, const QString &run
     }
 
     const int queryLetterCount = term.count();
-    for (Plasma::AbstractRunner *r : qAsConst(runnable)) {
+    for (Plasma::AbstractRunner *r : std::as_const(runnable)) {
         if (r->isMatchingSuspended()) {
             continue;
         }
+        // If this runner is loaded but disabled
+        if (!d->singleMode && d->disabledRunnerIds.contains(r->id())) {
+            continue;
+        }
         // The runners can set the min letter count as a property, this way we don't
         // have to spawn threads just for the runner to reject the query, because it is too short
         if (!d->singleMode && queryLetterCount < r->minLetterCount()) {
@@ -1017,12 +1025,6 @@ void RunnerManager::reset()
 
     d->searchJobs.clear();
 
-    if (d->deferredRun.isEnabled()) {
-        QueryMatch tmpRun = d->deferredRun;
-        d->deferredRun = QueryMatch(nullptr);
-        tmpRun.run(d->context);
-    }
-
     d->context.reset();
     Q_EMIT queryFinished();
 }
@@ -1036,14 +1038,14 @@ void RunnerManager::enableKNotifyPluginWatcher()
             if (groupName == QLatin1String("Plugins")) {
                 reloadConfiguration();
             } else if (groupName == QLatin1String("Runners")) {
-                for (auto *runner : qAsConst(d->runners)) {
+                for (auto *runner : std::as_const(d->runners)) {
                     // Signals from the KCM contain the component name, which is the X-KDE-PluginInfo-Name property
                     if (changedNames.contains(runner->metadata(RunnerReturnPluginMetaData).pluginId().toUtf8())) {
                         runner->reloadConfiguration();
                     }
                 }
             } else if (group.parent().isValid() && group.parent().name() == QLatin1String("Runners")) {
-                for (auto *runner : qAsConst(d->runners)) {
+                for (auto *runner : std::as_const(d->runners)) {
                     // If the same config group has been modified which gets created in AbstractRunner::config()
                     if (groupName == runner->id()) {
                         runner->reloadConfiguration();
